from typing import override
from astropy.units import Quantity, uV  # pyright: ignore[reportAttributeAccessIssue]
from astropy.units.physical import length, electrical_potential, time
from gwpy.time import Time
from h5py import File, Group

from backscattering_experimental_data.metadata.metadata import Metadata
from backscattering_experimental_data.utils import download_timeseries

from typing import TYPE_CHECKING

## AVOID CIRCULAR DEPENDENCY
if TYPE_CHECKING:
    from backscattering_experimental_data.component import (
        Component as ComponentData,
    )
    from backscattering_experimental_data.scatterer import (
        Scatterer as ScattererData,
    )


class Component(Metadata):
    """
    Metadata associated to a vda.package.Component
    """

    def __init__(
        self,
        name: str,
        calibration_factor: Quantity[length / electrical_potential],
    ):
        """
        Parameters
        ----------
        name : str
            Unique name of the component
        calibration_factor : astropy.units.Quantity[ length / electrical potential ]
            Calibration factor to correct movement data. Movement
            sensor give movement in Volt, and have to be corrected by a
            given factor, so that `real movement = calibration_factor *
            sensor data`
        Warnings
        --------
        It's an internal class, of whose instances are not produced via
        a direct call, but by using helper functions
        See Also
        --------
        bed.Component : Component class containing this metadata
        Examples
        --------
        >>> component_metadata = bed.MetadataComponent(
        ...     "BS",
        ...     1.0 * u.m / u.V,
        ... )
        """
        self.name = name
        self.calibration_factor = calibration_factor

    def fetch(
        self,
        start_time: Time,
        duration: Quantity[time],
    ) -> ComponentData | ScattererData:
        """
        Fetch a component from online data

        Construct a bed.Component object, containing all the data of
        this component
        Parameters
        ----------
        start_time : Time
            Starting time of the data
        duration : Quantity[time]
            Duration of the data
        Returns
        -------
        bed.Component : constructed Component from these metadata. It
        cannot be a bed.Scatterer, but setting it as a possible return
        type allows the subclass ScattererMetadata to be compatible with
        this definition
        """
        sensor = download_timeseries(
            "V1:Sc_{}_MIR_Z".format(self.name),
            start_time=start_time,
            duration=duration,
            unit=uV,  # in um, but the calibration factor correct the signal
        )
        return ComponentData(self, sensor=sensor)

    @override
    @staticmethod
    def from_hdf5(store: File | Group) -> "Component":
        for attribute in ["name", "calibration factor"]:
            if attribute not in store.attrs.keys():
                raise ValueError(
                    "no {} for an interferometer's component".format(
                        attribute
                    )
                )
        name = store.attrs["name"]
        calibration_factor = store.attrs["calibration factor"]

        error = (
            "interferometer's component {} should be a string, not {}"
        )
        if not isinstance(name, str):
            raise ValueError(error.format("name", type(name)))
        if not isinstance(calibration_factor, str):
            raise ValueError(
                error.format(
                    "calibration factor", type(calibration_factor)
                )
            )

        calibration_factor = Quantity(calibration_factor)

        return Component(
            name=name,
            calibration_factor=calibration_factor,
        )

    @override
    def to_hdf5(self, store: File | Group) -> Group | File:
        store.attrs["name"] = self.name
        store.attrs["calibration factor"] = (
            self.calibration_factor.to_string()
        )
        return store
