from astropy.units import Quantity, Unit, s, Hz  # pyright: ignore[reportAttributeAccessIssue]
from astropy.units.physical import time
from h5py import Dataset
from numpy import diff, mean
from requests import HTTPError

from gwpy.time import Time, to_gps
from gwpy.timeseries import TimeSeries

try:
    from virgotools.frame_lib import FrameFile, FrVect  # pyright: ignore[reportMissingImports]

    virgotool = True
except ImportError:
    virgotool = False
    pass


def dataset_to_timeseries(dataset: Dataset) -> TimeSeries:
    """
    Convert a h5df dataset to a TimeSeries keeping the units right
    Parameters
    ----------
    dataset : Dataset
        Dataset to convert
    Returns
    -------
    TimeSeries : TimeSeries with a good unit
    """
    unit = None
    if "units" in dataset.attrs.keys():
        units = dataset.attrs["units"]
        if isinstance(units, str):
            unit = Unit(units)
        else:
            raise ValueError(
                "cannot convert {} to a unit".format(units)
            )
    return TimeSeries(
        data=dataset[1],
        unit=unit,
        t0=dataset[0, 0],
        dt=mean(diff(dataset[0])),
    )


def download_timeseries(
    channel: str,
    start_time: Time,
    duration: Quantity[time],
    unit: Unit | None = None,
) -> TimeSeries:
    """
    Download a TimeSeries from an online channel
    Parameters
    ----------
    channel : str
        Channel name
    start_time : Time
        Starting time
    duration : Quantity[time]
        Duration
    unit : Unit | None
        Unit of the TimeSeries (default ot None)
    Notes
    -----
    It first try to use gwpy, and fallback to virgotools
    """
    try:
        return TimeSeries.get(
            channel=channel,
            start=to_gps(start_time),
            end=to_gps(start_time + duration),
        )
    except HTTPError:
        if virgotool:
            vect: FrVect | None = None  # pyright: ignore[reportPossiblyUnboundVariable]
            with FrameFile("raw") as ffl:  # pyright: ignore[reportPossiblyUnboundVariable]
                vect = ffl.getChannel(  # pyright: ignore[reportPossiblyUnboundVariable]
                    channel, to_gps(start_time), duration.to(s)
                )
            if vect is None:  # pyright: ignore[reportPossiblyUnboundVariable]
                raise RuntimeError(
                    "cannot get data vector from online data"
                )
            return TimeSeries(
                data=vect.data,  # pyright: ignore[reportPossiblyUnboundVariable]
                unit=unit,
                t0=start_time,
                sample_rate=vect.fsample * Hz,  # pyright: ignore[reportPossiblyUnboundVariable]
            )
        else:
            raise RuntimeError(
                "virgotools not available in this environment, cannot download online Virgo data"
            )
